import * as kernel from "@lumeweb/libkernel/kernel";
import {
  dnsClient,
  ethClient,
  handshakeClient,
  ipfsClient,
  networkRegistryClient,
  peerDiscoveryClient,
  swarmClient,
} from "./clients.js";
import { ed25519 } from "@lumeweb/libkernel";

document.addEventListener("DOMContentLoaded", () =>
  document.getElementById("go-button")?.addEventListener("click", () => {
    let input = (
      document.getElementById("address-bar") as HTMLInputElement
    ).value.trim();

    // If the input doesn't contain a protocol, assume it's http
    if (!input.match(/^https?:\/\//)) {
      input = `http://${input}`;
    }

    try {
      // Try to parse it as a URL
      const url = new URL(input);

      // Update the iframe's src attribute
      const iframe = document.getElementById(
        "web-content",
      ) as HTMLIFrameElement;
      iframe.src = `/browse/${url.hostname}${url.pathname}${url.search}${url.hash}`;
    } catch (e) {
      // Handle invalid URLs here, if needed
      console.error("Invalid URL:", e);
    }
  }),
);

let BOOT_FUNCTIONS: (() => Promise<any>)[] = [];

async function boot() {
  await kernel.init();
  await kernel.login(ed25519.utils.randomPrivateKey());

  const reg = await navigator.serviceWorker.register("/sw.js");
  await reg.update();

  await kernel.serviceWorkerReady();

  BOOT_FUNCTIONS.push(
    async () =>
      await swarmClient.addRelay(
        "2d7ae1517caf4aae4de73c6d6f400765d2dd00b69d65277a29151437ef1c7d1d",
      ),
  );

  // IRC
  BOOT_FUNCTIONS.push(
    async () =>
      await peerDiscoveryClient.register(
        "zdiN5eJ3RfHpZHTYorGxBt1GCsrGJYV9GprwVWkj8snGsjWSrptFm8BtQX",
      ),
  );
  BOOT_FUNCTIONS.push(
    async () => await networkRegistryClient.registerType("content"),
  );
  BOOT_FUNCTIONS.push(
    async () => await networkRegistryClient.registerType("blockchain"),
  );
  BOOT_FUNCTIONS.push(async () => await handshakeClient.register());
  BOOT_FUNCTIONS.push(async () => await ethClient.register());
  BOOT_FUNCTIONS.push(async () => await ipfsClient.register());

  const resolvers = [
    "zdiJdDdBJWAdYFTcRa9So5TQQ9f1pYMiMy4dqYcKp9imomQtR11LJUyJyV", // CID
    "zdiKvnZYNjDqXaM8uF3pGEs7Tt6jqGc7t7M4eqbvJwpkTnrZymncfUW9Cj", // ENS
    "zrjEH3iojPLr7986o7iCn9THBmJmHiuDWmS1G6oT8DnfuFM", // HNS
  ];

  for (const resolver of resolvers) {
    BOOT_FUNCTIONS.push(async () => dnsClient.registerResolver(resolver));
  }

  await bootup();

  await Promise.all([
    ethClient.ready(),
    handshakeClient.ready(),
    ipfsClient.ready(),
  ]);

  document.getElementById("booting")!.style.display = "none";
  (document.getElementById("address-bar") as HTMLInputElement).disabled = false;
  (document.getElementById("go-button") as HTMLButtonElement).disabled = false;
}

async function bootup() {
  for (const entry of Object.entries(BOOT_FUNCTIONS)) {
    await entry[1]();
  }
}

document.addEventListener("DOMContentLoaded", boot);
